//
//  NSData+util.m
//  NSData+util (Released under MIT License)
//
//  Created by digdog on 1/14/09.
//  Copyright (c) 2009 Ching-Lan 'digdog' HUANG. http://digdog.tumblr.com
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//  
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
//  

#import "NSData+util.h"
#import <CommonCrypto/CommonDigest.h>
#import <CommonCrypto/CommonCryptor.h>

static int asciitable[128] = {
99,99,99,99, 99,99,99,99, 99,99,99,99, 99,99,99,99,
99,99,99,99, 99,99,99,99, 99,99,99,99, 99,99,99,99,
99,99,99,99, 99,99,99,99, 99,99,99,99, 99,99,99,99,
0, 1, 2, 3,  4, 5, 6, 7,  8, 9, 99,99, 99,99,99,99, // 0..9
99,10,11,12, 13,14,15,99, 99,99,99,99, 99,99,99,99, // A..F
99,99,99,99, 99,99,99,99, 99,99,99,99, 99,99,99,99,
99,10,11,12, 13,14,15,99, 99,99,99,99, 99,99,99,99, // a..f
99,99,99,99, 99,99,99,99, 99,99,99,99, 99,99,99,99
};

@implementation NSData (NSData_Hex)

- (NSData *) hex2raw {
	// Based on Erik Doernenburg's NSData+MIME.m
	const char *source, *endOfSource;
	NSMutableData *decodedData;
	char *dest;
	
	source = [self bytes];
	endOfSource = source + [self length];
    decodedData = [NSMutableData dataWithLength:[self length]];
    dest = [decodedData mutableBytes];

	while (source < endOfSource) {
		if (isxdigit(*source) && isxdigit(*(source+1))) {
			*dest++ = asciitable[(int)*source] * 16 + asciitable[(int)*(source+1)];
			source += 2;			
		} else
			return nil;
	}
	
	[decodedData setLength:(unsigned int)((void *)dest - [decodedData mutableBytes])];
	
	return decodedData;
}

- (NSData *) decode {
	unsigned char *buffer = NULL;
	unsigned char hashedChars[CC_SHA1_DIGEST_LENGTH];
	CCCryptorStatus status;
	CCCryptorRef decrypt_ref;
	size_t decrypted_length = 0, padded_length = 0;
	
	if ([self length] < 48) {
		NSLog(@"text not long enough");
		return nil;
	}
	
	NSData *ct = self;
	NSData *h1 = ct;
	NSData *iv = [h1 subdataWithRange:NSMakeRange(0, 8)];
	NSData *h4 = [ct subdataWithRange:NSMakeRange(20, [ct length] - 20)];
	
	NSMutableData *enc = [NSMutableData dataWithLength:([ct length] - 40)];
	[enc setData:[ct subdataWithRange:NSMakeRange(40, [ct length] - 40)]];
	
	NSMutableData *ht = [NSMutableData dataWithLength:20];
	[ht setData:[h1 subdataWithRange:NSMakeRange(0, 20)]];
	buffer = [ht mutableBytes];
	
	buffer[19]++;
	CC_SHA1([ht bytes], [ht length], hashedChars);
	NSData *h2 = [NSData dataWithBytes:hashedChars length:CC_SHA1_DIGEST_LENGTH];
	
	buffer[19] += 2;
	CC_SHA1([ht bytes], [ht length], hashedChars);
	NSData *h3 = [NSData dataWithBytes:hashedChars length:CC_SHA1_DIGEST_LENGTH];
	buffer = NULL;
	
	NSMutableData *key = [NSMutableData dataWithLength:24];
	[key setData:[h2 subdataWithRange:NSMakeRange(0, 20)]];
	[key appendData:[h3 subdataWithRange:NSMakeRange(0, 4)]];
	
	// Parity Check
	CC_SHA1([enc bytes], [enc length], hashedChars);
	NSData *new_ht = [NSData dataWithBytes:hashedChars length:CC_SHA1_DIGEST_LENGTH];		
	if (![new_ht isEqualToData:[h4 subdataWithRange:NSMakeRange(0, 20)]]) {
	    //NSLog(@"Parity check failed");
		return nil;
	}
	
	// CCCryptorCreate
	status = CCCryptorCreate(kCCDecrypt, kCCAlgorithm3DES, 0, [key bytes], [key length], [iv bytes], &decrypt_ref);
	
	if (status != kCCSuccess) {
		//NSLog(@"CCCryptorCreate failed");
		return nil;
	}
	
	// CCCryptorUpdate
	buffer = [enc mutableBytes];
	status = CCCryptorUpdate(decrypt_ref, buffer, [enc length], buffer, [enc length], &decrypted_length);
	
	if (status != kCCSuccess) {
		//NSLog(@"CCCryptorUpdate failed");
		return nil;
	}
	
	// CCCryptorFinal
	status = CCCryptorFinal(decrypt_ref, buffer + decrypted_length, [enc length] - decrypted_length, &padded_length);
	
	if (status != kCCSuccess) {
		//NSLog(@"CCCryptorFinal failed");
		return nil;
	}
	
	CCCryptorRelease(decrypt_ref);
	
	// Plain
	int plainLength = [enc length] - buffer[([enc length]-1)];
	NSMutableData *plain = [NSMutableData dataWithLength:plainLength];
	[plain setData:[enc subdataWithRange:NSMakeRange(0, plainLength)]];
	
	return plain;	
}

@end