//
//  RootViewController.m
//  Grupa Reveal (Released under New BSD License)
//
//  Created by Ching-Lan 'digdog' HUANG on 1/23/09.
//  Copyright (c) 2009 Ching-Lan 'digdog' HUANG. All rights reserved. http://digdog.tumblr.com
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//  * Neither the name of the digdog software nor the
//    names of its contributors may be used to endorse or promote products
//    derived from this software without specific prior written permission.
// 
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL CHING-LAN 'digdog' HUANG BE LIABLE FOR ANY
//  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import "RootViewController.h"
#import "Grupa_RevealAppDelegate.h"
#import "ScrambledCell.h"
#import "InfoCell.h"
#import "NSData+util.h"


@implementation RootViewController

@synthesize tableView;
@synthesize statusString;

- (void)viewDidLoad {
    [super viewDidLoad];
	self.navigationItem.title = @"Grupa Reveal";
	self.navigationItem.prompt = @"Recovering Cisco IPSec VPN Group Password";
	statusString = @"Input scrambled hexadecimal text to decode";
	needReload = NO;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)dealloc {
	[tableView release];
    [super dealloc];
}

#pragma mark Private methods

- (UITextView *)createScrambledTextView {	
	UITextView *textView = [[[UITextView alloc] initWithFrame:CGRectZero] autorelease];
	
    textView.font = [UIFont fontWithName:@"Courier" size:15];
    textView.textColor = [UIColor blackColor];
    textView.backgroundColor = [UIColor whiteColor];
    textView.delegate = self;
	textView.text = @"954B72E9D5D2203AD830793B1D130A913579674FDC453E5B31FFF6C6872379A3B6E3A809484CD2E89B5D581C54FC6BC4";
	textView.keyboardType = UIKeyboardTypeNamePhonePad;
	textView.returnKeyType = UIReturnKeyDone;
	textView.autocapitalizationType = UITextAutocapitalizationTypeAllCharacters;
	textView.autocorrectionType = UITextAutocorrectionTypeNo;
	
	placeHolder = YES;
	
	return textView;	
}

- (void)decode:(NSString*)ct {
	if ([ct length]) {
		NSData *cipher = [[ct dataUsingEncoding:NSASCIIStringEncoding] hex2raw];
		if (!cipher) {
			statusString = @"Text length should be even";		
		} else {
			if ([cipher length] < 48) {
				statusString = @"Text length is not enough";
			} else {
				NSData *plain = [cipher decode];
				if (plain) {
					statusString = @"Success";
					UIAlertView * errorAlert = [[UIAlertView alloc] initWithTitle:@"Your Group Password is" message:[NSString stringWithCString:[plain bytes] encoding:NSUTF8StringEncoding] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
					[errorAlert show];			
				}
				else
					statusString = @"Unable to recover the scrambled text";
			}
		}		
	} else {
		statusString = @"Input scrambled hexadecimal text to decode";
	}
}

#pragma mark UITextViewDelegate methods
- (void)textViewDidBeginEditing:(UITextView *)textView {	
	if (placeHolder) {
		textView.text = @"";
		placeHolder = NO;
	}	
	
	statusString = @"Input scrambled hexadecimal text to decode";
	[self.tableView reloadData];
}

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text {
	
	if ([text canBeConvertedToEncoding:NSASCIIStringEncoding]) {
		const char *cStr = [text cStringUsingEncoding:NSASCIIStringEncoding];

		if (cStr[0] == '+')
			return YES;
		
		if (isprint(cStr[0]) && !isxdigit(cStr[0]))
			return NO;
	} else
		return NO;
	
	if ([text isEqualToString:@"\n"]) {
		[textView resignFirstResponder];
		[self decode:textView.text];
		needReload = YES;
		[self.tableView reloadData];
		return NO;
	}
	
	return YES;
}

- (void)textViewDidChange:(UITextView *)textView {
	NSRange foundPlus = [textView.text rangeOfString:@"+"];
	if (foundPlus.location != NSNotFound)
		textView.text = [textView.text stringByReplacingCharactersInRange:foundPlus withString:@"0"];
}

#pragma mark UITextViewDelegate methods

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	if (section == 0)
		return @"enc_GroupPwd in your .pcf file";	// section 1
	return @"Disclaimer";						// section 2
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
	if (section == 1) 
		return 160.0f;	// section 2
	return 0.0f;		
}
- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section {
	if (section == 1) {
		UIView *footerView = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 300, 160)] autorelease];
		UILabel *disclaimerLabel = [[[UILabel alloc] initWithFrame:CGRectMake(10, 0, 300, 160)] autorelease];
		disclaimerLabel.font = [UIFont systemFontOfSize:9];
		disclaimerLabel.textColor = [UIColor grayColor];
		disclaimerLabel.backgroundColor = [UIColor clearColor];
		disclaimerLabel.numberOfLines = 15;
		disclaimerLabel.adjustsFontSizeToFitWidth = YES;
		disclaimerLabel.text = @"THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.";
		[footerView addSubview:disclaimerLabel];
		return footerView;
	}
	return nil;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
	if (indexPath.row == 0) 
		return 80;	// row 1
	return 34;		// row 2
}

#pragma mark UITableViewDataSource methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	if (section == 0)
		return 2;	// section 1
	return 0;		// section 2
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)aTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier;
	UITableViewCell *cell;
	
	if (indexPath.row == 0) {
		CellIdentifier = @"ScrambledCell";
	} else if (indexPath.row == 1) {
		CellIdentifier = @"InfoCell";
	}
	
	cell = [aTableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
		if (indexPath.row == 0 && !needReload) {
			cell = [[[ScrambledCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier] autorelease];
		} else if (indexPath.row == 1) {
			cell = [[[InfoCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier] autorelease];
		}
	}
    
	if (indexPath.row == 0 && !needReload) {
		((ScrambledCell *)cell).scrambledTextView = [self createScrambledTextView];
	} else if (indexPath.row == 1) {
		((InfoCell *)cell).plainLabel.text = statusString;
	}
	
    return cell;
}

@end

